#include <stdint.h>
#include <cpu.h>
#include <ports.h>

#define PC_SPKR 0x61
#define PIT_CH2 0x42
#define PIT_CMD 0x43

#define PIT_FREQ 1193182

#define APIC_TIMER_DIVIDER 0x3E0
#define APIC_TIMER_INIT 0x380
#define APIC_TIMER_CURRENT 0x390

volatile uint32_t apic_ticks_per_us = 1;

static void pit_setup(uint8_t ms) {
  // Disable the pc speaker output
  uint8_t spkr = inb(PC_SPKR);
  spkr &= ~0x1;
  outb(PC_SPKR, spkr);

  // Set up PIT for a single interrupt on CH2
  outb(PIT_CMD, 0x80 | 0x30);

  // Set up CH2 for counting down from a number of ms
  uint64_t count = PIT_FREQ * ms/1000;
  outb(PIT_CH2, count & 0xFF);
  outb(PIT_CH2, (count >> 8) & 0xFF);
}

static void pit_wait() {

  // Enable pc speaker
  uint8_t spkr = inb(PC_SPKR);
  spkr |= 0x1;
  outb(PC_SPKR, spkr);

  // Wait until there's a speaker output
  while(!(inb(PC_SPKR) & 0x20));
}

static uint64_t calibrate_apic_timer() {
  pit_setup(10);

  APIC_REG(APIC_TIMER_DIVIDER) = 0xB;
  APIC_REG(APIC_LVT(LVT_TIMER)) = 1<<16; // Mask
  APIC_REG(APIC_TIMER_INIT) = 0xFFFFFFFF;

  pit_wait();

  uint32_t ticks = APIC_REG(APIC_TIMER_CURRENT);
  return (0xFFFFFFFF-ticks)/10/1000;
}

void reset_apic_timer(uint64_t us) {
  APIC_REG(APIC_TIMER_DIVIDER) = 0xB;
  APIC_REG(APIC_TIMER_INIT) = us*current_cpu->timer_freq;
  APIC_REG(APIC_LVT(LVT_TIMER)) = 0x0 | IRQ_INTERRUPT(IRQ_TIMER);
}

volatile int ctr = 0;

registers *apic_timer_handler(registers *r) {
  current_cpu->timer_ticks++;
  reset_apic_timer(1000);
  irq_ack();
  return r;
}

void timer_init() {
  current_cpu->timer_ticks = 0;
  current_cpu->timer_freq = calibrate_apic_timer();
  bind_interrupt(IRQ_INTERRUPT(IRQ_TIMER), apic_timer_handler);
  reset_apic_timer(1000);
}